#!/usr/bin/perl
# CONFIG: Change the first line to indicate the Perl binary on your system.
#          Try "which perl", "whereis perl", or "find / -name perl -print"

############################################################## -*- Perl -*- ###
#                                                                          
#     AccessWatch v2.4 - Traffic analysis for World Wide Web sites          
#      Copyright 1994-2006 David Maher. All rights reserved.         
#      http://accesswatch.com/                                              
#                                                                           
#     Contact Information:                                                  
#        AccessWatch                                                          
#        PO Box 60, Shady Side, MD 20764 USA
#                                                                             
#     Please direct comments to:                                              
#        http://accesswatch.com/comments/
#                                                                             
#     Please read the included license agreement (README) before          
#      using this program. Your use of this software indicates your           
#      acceptance of the license agreement and warranty.                      
#                                                                             
#     If you modify this code, mail changes@accesswatch.com.
#     We encourage modifications by registered users, but 
#     redistribution is prohibited. Just write us first. We'd 
#     appreciate your ideas and suggestions.                                  
#
#  Credits and thanks for great ideas:
#
#   A huge thank you to Paul Blackman for his work on hourly 
#   server statistics, including a table of hourly accesses/hour.          
#                                                                           
#   Thanks to Jeff Boulter for access error exclusion code, 
#   referer log parsing tips, and for his many suggestions  
#   and debugging help.                                                     
#                                                                           
#   Many thanks for suggestions and code tweaks:                              
#
#      Chris Brown          
#      Ron Gery             
#      Lars Bo Eriksen      
#      Tieshy Lin           
#      Chris Derossi        
#      Arthur Hagen         
#      Eric Ruck            
#      Tim Dobbelaere       
#      Jean-Francois Monnet 
#      Monte Ohrt     
#      Guillaume Martin
#
###############################################################################

($perltest=$0) =~ tr/[A-Z]/[a-z]/;   # Perl is not installed correctly if you received this error. See http://www.perl.com/ for more information about Perl 5.

($base=$0) =~ s/[A-z0-9,\.,\-]*$//;  # if you have trouble, define $base as
                                     #  the full directory pathname that 
                                     #  contains the accesswatch script.

if ($DNSlookups) {
    use Socket;    # DNS Lookups On
}

require $base.'lib/timelocal.pl' 
    || die "Error: Trouble locating timelocal.pl "
             . "- contact your system administrator $!\n";

require $base.'lib/global.lib'
    || die "Error: Global file not found $!\n"; 

require $base.'lib/db.lib' 
    || die "Error: Database access file not found $!\n"; 

require $base.'lib/time.lib' 
    || die "Error: Time library file not found $!\n"; 

##############################################################################

#-----------------------------------------------------------------------------#
#  AccessWatch function - Main
#    Purpose  : Coordinates everything else...
#-----------------------------------------------------------------------------#

MAIN: {

    my($beginS, $endS, $timeperiod);

    if ($opt_e) {
	print "Erasing AW database files...\n" if $verbose;
	&removeFiles;
    }

    if ($opt_s) {
	($beginS, $endS) = &ReturnBeginEndMonth($opt_s);	
	$timeperiod = "$opt_s";
    }
    elsif ($opt_w) {
	($beginS, $endS) = ($weekS, $todayS);
	$timeperiod = "the last week";
    }
    elsif ($opt_m) {
	($beginS, $endS) = ($monthS, $todayS);
	$timeperiod = "the last month";
    }
    elsif ($opt_y) {
	($beginS, $endS) = ($yearS, $todayS);
	$timeperiod = "the last year";
    }
    elsif ($opt_d) {
	($beginS, $endS) = ($yestrS, $todayS);
	$timeperiod = "the last day";
    }
    elsif ($opt_t) { # The whole log option!!
	($beginS, $endS) = (0, time);
	$timeperiod = "the entire log";
    }
    else {
	($beginS, $endS) = (&LastRun, time);
	$timeperiod = "the log activity since the last run";
    }

    print "AccessWatch v$version starting, parsing $timeperiod.\n"
	if $verbose;

    # Sure, you are clever. But we all like *honest* clever people.
    if (0 && ! -f $base.'lib/registered') {

        # Visit https://www.accesswatch.com/license/order/
        #  to register instantly and inexpensively.

	$unregistered = 1;	
	open (REG, "<${base}lib/register") || die "Could not find registration file: $!\n";
	my $trialtime = <REG>;
	close REG;

	if ((time - $trialtime) > 1209600) {

	    print "\n\n\n"
		. "!" x 70 . "\n"
		. " Your AccessWatch 14-day trial is complete. Please visit\n"
		. " https://www.accesswatch.com/license/order/ "
		    . "to register now.\n"
			. "!" x 70 . "\n\n";

	    if ((time - $trialtime) > 1382400) {
		exit;
	    }
	}
	else {
	    print "\n"
		. "-> This is an 14-day evaluation copy of AccessWatch. Please visit\n"
		    . "-> https://www.accesswatch.com/license/order/ to register.\n"
			. "\n";
	}
    }

    my($firstDateFoundS, $lastDateFoundS);

    if (-f $accessLog) {

	# a file

	open (LOG, "<$accessLog") 
	    || die "Access Log Error: Couldn't open $accessLog $!\n";

	($firstDateFoundS, $lastDateFoundS) 
	    = &ProcessAccessLog($beginS, $endS);
    
	close (LOG);

	if ($firstDateFoundS && $lastDateFoundS > $firstDateFoundS) {
	    &RecordRange($firstDateFoundS, $lastDateFoundS);
	}

    }
    elsif (-d $accessLog) {

	# a directory with multiple logs

	opendir (LOGDIR, "$accessLog") 
	    || die "Could not open logfile directory $accessLog: $!\n";
	my(@files) = grep ! /^\./, (readdir(LOGDIR));
	closedir LOGDIR;
	
	foreach (@files) {	
	    if (-r "$accessLog/$_" && $_ !~ /^error/i && $_ !~ /^ref/i
		&& $_ !~ /^agent/i) {

		print "Reading $accessLog/$_.\n";

		my($standardfile);

		if ($_ =~ /.gz$/i || $_ =~ /.zip$/i || $_ =~ /.Z/i) {
		    open (LOG, "$catcompresscommand $accessLog/$_|")
 		 || die "Access Log Error: Couldn't open $accessLog/$_: $!\nMake sure that the \$catcompresscommand variable is set correctly and the command is in your path.\n";
		    $standardfile = 0;
		}
		else {
		    open (LOG, "<$accessLog/$_") 
		 || die "Access Log Error: Couldn't open $accessLog/$_: $!\n";
		    $standardfile = 1;
		}
		
		($firstDateFoundS, $lastDateFoundS) 
		    = &ProcessAccessLog($beginS, $endS);
		
		close (LOG);

		if ($firstDateFoundS && $lastDateFoundS > $firstDateFoundS) {
		    &RecordRange($firstDateFoundS, $lastDateFoundS);
		}

	    }
	    else {
		print "Could not read $accessLog/$_. Skipping...\n";
	    }
	}
    }
    
    print "Log file parsing complete.\n" 
	if $verbose;

    exit 0;
}


#-----------------------------------------------------------------------------#
#  AccessWatch function - ProcessAccessLog
#    Purpose  : Scans through access log and picks out the appropriate accesses
#-----------------------------------------------------------------------------#
sub ProcessAccessLog {

#LOG:$remote, $d1, $d2, $date, $tz, $method, $page, $protocol, $protnum1, $size

    my($beginTime, $endTime) = ($_[0], $_[1]);

    my($firstDateFoundS) = &FastSearch($beginTime);

#    if (!$firstDateFoundS) {
#	print "Problems with the log file - search date not found.\n";
#	return ();
#    }

    my(@hourHosts, $currentHour, $lastDateFoundS);

    my($sitedomain) = $siteBaseURL;
    
    $sitedomain =~ /:\/\/[w]{3}?([^\/]+)/;
    
    $sitedomain = $1;
    
    $sitedomain =~ s/^\.(.*)/$1/g;
 
    my($remote, $dash1, $dash2, $date, $tz, 
	  $method, $page, $protocol, $protnum1, $size, $ref, $agent);

    my($hour, $min, $sec, $lday, $pday, $lmonth, $lyear, $datecode);

    my($lineTimeLF, $futtimes);
    my($beginTimeLF) = &FormatLogTime($firstDateFoundS);
    my($endTimeLF) = &FormatLogTime($endTime);

    print "Begin time: $beginTimeLF\tEnd time  : $endTimeLF\n" if $verbose;
    
    $| = 1; 
    
    print "Parsing access log..." if $verbose;

    while (<LOG>) {		
	
	$lineTimeLF = $_;
	$lineTimeLF =~ s/^[^\]]+\[([^\]]+).*$/$1/goi;

	if (&CompareLogTime($beginTimeLF, $lineTimeLF) < 0
	    || &ParsedRange(&ReturnSecondsFromLogDate($lineTimeLF))) {

	    # Either out of range or already in the DB.

	    next;
	}
	elsif (&CompareLogTime($endTimeLF, $lineTimeLF) > 0) {

	    # End processing if we've overshot the range. 

	    $futtimes++;

	    if ($futtimes > 4000) {
		
		# Take into account some server log buffering.

		last;
	    }
	    next;
	}
	
        $periodStats{'serverCount'}++;
	
	next unless 
(/^(\S+) \S+ \S+ \[(\S+):(\S+):(\S+):(\S+) \S+\] \"([^\"]+)\" (\S+) (\S+)/o);

	($remote, $date, $page, $protnum1, $size) = ($1, $2, $6, $7, $8);
        ($hour, $min, $sec) = ($3, $4, $5);
	($lday, $lmonth, $lyear) = split('/', $date);

        $newdatecode = sprintf("%04d%02d%02d%02d", 
				$lyear, $monthTable{$lmonth}, $lday, $hour);

        if ($newdatecode ne $datecode) {
	    print ".$newdatecode." if ($pday ne $lday && $verbose);

	    foreach $oldcode (keys %AWdata) {
		if (abs($oldcode - $datecode) > 1 && $AWdata{$oldcode}) { 
                    # data more than 1 hour old - takes care 
                    # of log file caching by busy servers.
#		    print "CACHE Writing Stale $oldcode\n";
		    &writeFile($oldcode);
		    delete $AWdata{$oldcode};
		}
	    }

	    if (! %{ $AWdata{$newdatecode} }) {
#		print "CACHE Reading New $newdatecode\n";
		&readFile($newdatecode);
	    }
	    $datecode = $newdatecode;
        }
        
#        print ".$lmonth/$lday/$lyear." if ($pday ne $lday && $verbose);
        $pday = $lday;

        $page = &FilterURL($page)
                       if (int($protnum1) < 400);

        if (($remote =~ /[\d]+\.[\d]+\.[\d]+\.[\d]+/) && $DNSlookups) {
	    $remote = lookuphost($remote); 
	}
	
        $remote .= ".$orgdomain" if ($remote !~ /\./o);

        #takes care of those internal accesses that do not get fully 
        # qualified in the log name -> name.orgname.ext

	next unless $page =~ /$includeURL/io && $page ne ''
           && !($remote =~ /$excludeHosts/io && $excludeHosts ne "");

        # Agent and Ref Stats
        if (m|\"([^\"]*)\" \"([^\"]*)\"$|o) {

	    ($ref, $agent) = ($1, $2);
	    if ($ref eq '' || $ref eq '-' || $ref =~ /$sitedomain/i) {
		undef $ref;
	    }
	    if ($agent eq '' || $agent eq '-') {
		undef $agent;
	    }
	}
        else {
	    undef $ref;
	    undef $agent;
        }
	
        # Compress proxies
	$remote =~ tr/[A-Z]/[a-z]/;
	$remote =~ s/.*.proxy.aol.com/www.proxy.aol.com/oi;
	$remote 
            =~ s/piweb[a-z][0-9][a-z]-ext.prodigy.com/piweb-ext.prodigy.com/oi;

        $AWdata{$datecode}{'codes'}{$protnum1}++;
        $AWdata{$datecode}{'hosts'}{$remote}++;
        
        if (int($protnum1) < 400) {
	    $AWdata{$datecode}{'pages'}{$page}++;
	    $AWdata{$datecode}{'filesize'}{$page} += $size 
		if ($size =~ /^\d+$/);
	}
        else {
	    $AWdata{$datecode}{'errors'}{"$page-$protnum1"}++;
        }

        $AWdata{$datecode}{'browsers'}{$agent}++ if $agent;
	$AWdata{$datecode}{'references'}{"$ref~~~$page"}++ if $ref;
    }				

    # flush the cache
    foreach $hour (keys %AWdata) {

        &writeFile($hour);
    }

    print " done.\n" if $verbose;
   
    $| = 1; 

    return ($firstDateFoundS, 
    &ReturnSecondsFromLogDate("$date:$hour:$min:$sec"));
}


#-----------------------------------------------------------------------------#
#  AccessWatch function - FastSearch
#    Purpose  : Scans quickly from the bottom of the access log to start
#                   of the given day.
#-----------------------------------------------------------------------------#

sub FastSearch {

    my($searchTime) = $_[0];

    if ($verbose) {
	$| = 1; print "Scanning for start of current log period..."; $| = 0;
    }

    if ($searchTime == 0) {
	seek(LOG, 0, 0); <LOG>; 
	$_ = <LOG>;
	seek(LOG, 0, 0);
	<LOG>;
	print " done.\n" if $verbose;
	return &ReturnSecondsFromLogDate($_);
    }

    my($searchTimeLF) = &FormatLogTime($searchTime);
    my($lineTimeLF) = "00/Jan/0000:00:00:00";

    my($avgDayGrowth) = 10000 * $hour + 
	(10000 * 24 * ($todayS - $searchTime)/(24*60*60)) + 1000;
    my($jumpsize) = $avgDayGrowth/5;

    my($logsize) = (-s LOG);
    my($logoffset, $logPosDiff, $jumps) = 0;

    $logoffset = sprintf("%d", $logsize - $avgDayGrowth)
	if ($logsize > $avgDayGrowth);

    seek (LOG, $logoffset, 0);
    <LOG>;

    search: while (<LOG>) {

	last search if ($logoffset < 0);

	$lineTimeLF = $_;
	$lineTimeLF =~ s/^[^\]]+\[([^\]]+).*$/$1/gi;

	$logPosDiff = &CompareLogTime($searchTimeLF, $lineTimeLF);

	if ($logPosDiff > 0) {
	    $logoffset = $logoffset - $jumpsize;
	}
	elsif ($logPosDiff < 0) {
	    forwardScan: while (<LOG>) { 
		$lineTimeLF = $_;
		$lineTimeLF =~ s/^[^\]]+\[([^\]]+).*$/$1/gi;     
		$logPosDiff = &CompareLogTime($searchTimeLF, $lineTimeLF);
		last forwardScan if ($logPosDiff >= 0);
	    }
	    last search;
	}
	else { last search; }

	$jumps++;

	if ($verbose && $jumps % 100 == 1) {
	    $| = 1;
	    print ".";
	    $| = 0;
	}

	seek (LOG, $logoffset, 0);
	<LOG>;
    }
    print " done.\n" if $verbose;
    
    return &ReturnSecondsFromLogDate($_);
}


#-----------------------------------------------------------------------------#
#  AccessWatch function - FilterURL
#    Purpose  : Takes a URL as an argument and decodes/trims it.
#-----------------------------------------------------------------------------#

sub FilterURL {

    my($url) = $_[0];

    $url =~ s/^\"//;
    $url =~ s/\"$//;                 # Remove leading and trailing quotes
    $url =~ s/^\w+\s//;              # Erase first HEAD GET POST PUT
    $url =~ s/^(\S+)\s?\S+$/$1/;     # Get rid of HTTP...
    $url =~ s/^-$/$1/;               # Erase '-' requests

    $url =~ tr/+/ /;
    $url =~ s/([^\?]+)\?.*/$1/o;        #strip last '?' from GET requests
    $url =~ s/([^\.]+)[\.]+$/$1/o;      #strip last '.'
    $url =~ s/$defaultPage$//o;
    $url =~ s/([^\/]+)[\/]+$/$1/o;      #strip last '/'
    $url =~ s/([^\#]+)\#.*/$1/o;        #strip last '#' from label access
    $url =~ s/(\/)[\/]+/$1/og;

    $url =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/ego;

    $url = "\/" if ($url eq "");
    return $url;
}


__END__


