#!/usr/bin/perl
# CONFIG: Change the first line to indicate the Perl binary on your system.
#          Try "which perl", "whereis perl", or "find / -name perl -print"

############################################################## -*- Perl -*- ###
#                                                                          
#     AccessWatch v2.4 - Traffic analysis for World Wide Web sites          
#      Copyright 1994-2006 David Maher. All rights reserved.         
#      http://accesswatch.com/                                              
#                                                                           
#     Contact Information:                                                  
#        AccessWatch                                                          
#        PO Box 60, Shady Side, MD 20764 USA                               
#                                                                             
#     Please direct comments to:                                              
#        http://accesswatch.com/comments/
#                                                                             
#     Please read the included license agreement (README) before          
#      using this program. Your use of this software indicates your           
#      acceptance of the license agreement and warranty.                      
#                                                                             
#     If you modify this code, mail changes@accesswatch.com.
#     We encourage modifications by registered users, but 
#     redistribution is prohibited. Just write us first. We'd 
#     appreciate your ideas and suggestions.                                  
#
#  Credits and thanks for great ideas:
#
#   A huge thank you to Paul Blackman for      
#   his work on hourly server statistics, including a table of hourly       
#   accesses/hour.                                                          
#                                                                           
#   Thanks to Jeff Boulter for access error        
#   exclusion code, referer log parsing tips, and for his many suggestions  
#   and debugging help.                                                     
#                                                                           
#   Many thanks for suggestions and code tweaks:                              
#
#      Chris Brown         
#      Ron Gery            
#      Lars Bo Eriksen     
#      Tieshy Lin          
#      Chris Derossi       
#      Arthur Hagen        
#      Eric Ruck           
#      Tim Dobbelaere      
#      Jean-Francois Monnet 
#      Monte Ohrt     
#      Guillaume Martin
#                                                                           
###############################################################################

($base=$0) =~ s/[A-z0-9,\.,\-]*$//;  # if you have trouble, define $base as
                                     #  the full directory pathname that 
                                     #  contains the accesswatch script.

require $base.'lib/timelocal.pl' 
    || die "Error: Trouble locating timelocal.pl "
             . "- contact your system administrator $!\n";

require $base.'lib/global.lib'
    || die "Error: Global file not found $!\n"; 

require $base.'lib/report.lib' 
    || die "Error: Reporting file not found $!\n"; 

require $base.'lib/table.lib' 
    || die "Error: Table generation file not found $!\n"; 

require $base.'lib/db.lib' 
    || die "Error: Database access file not found $!\n"; 

require $base.'lib/time.lib' 
    || die "Error: Time library file not found $!\n"; 

##############################################################################

#-----------------------------------------------------------------------------#
#  AccessWatch function - Main
#    Purpose  : Coordinates everything else...
#-----------------------------------------------------------------------------#

MAIN: {

    print "Accesswatch v$version Report Generator - $siteName\n" if $verbose || $opt_p;


    if ($opt_p) {
	&DisplayRange;
	exit;
    }

    if ($opt_r) {
	$timeperiod = "$begin - $end";
    }
    elsif ($opt_s) {
	($begin, $end) = &ReturnBeginEndMonth($opt_s);	
	$begin = &SecondsToHourIndex($begin);
	$end = &SecondsToHourIndex($end);
	$timeperiod = "$opt_s";
    }
    elsif ($opt_w) {
	($begin, $end) = (&SecondsToHourIndex($weekS), 
			  &SecondsToHourIndex($todayS));
	$timeperiod = "the last week";
    }
    elsif ($opt_m) {
	($begin, $end) = (&SecondsToHourIndex($monthS), 
			  &SecondsToHourIndex($todayS));
	$timeperiod = "the last month";
    }
    elsif ($opt_y) {
	($begin, $end) = (&SecondsToHourIndex($yearS), 
			  &SecondsToHourIndex($todayS));
	$timeperiod = "the last year";
    }
    elsif ($opt_d) {
	($begin, $end) = (&SecondsToHourIndex($yestrS), 
			  &SecondsToHourIndex($todayS));
	$timeperiod = "the last day";
    }
    else { # The whole log option!!
	($begin, $end) = (0, &SecondsToHourIndex(time));
	$timeperiod = "all recorded dates";
    }

    print "Reading stats summary for $timeperiod...\n" if $verbose;
    &openTables(0, $begin, $end);

    if (! -d $reportBase) {
	if (mkdir ($reportBase, 0755)) {
	    chmod 0755, $reportBase;
	}
	else {
	    die "Could not create report directory $reportBase: $!\n";
	}
    }
    if (! -d "$reportBase/img") {

	mkdir "$reportBase/img", 0755;
	print "Copying report image files...\n" if $verbose;

	use File::Copy;
	
	opendir(IMG, "${base}report/img") 
	    || die "Could not open img dir: $!\n";
	my(@images) = grep ! /^\./, readdir(IMG);
	closedir(IMG);
	foreach (@images) {
	    copy("${base}report/img/$_", "$reportBase/img/$_")
		|| die "Could not copy ${base}report/img/$_: $!\n";
	    chmod 0644, "$reportBase/img/$_";
	}
	chmod 0755, "$reportBase/img";
    }
    

    if (%AWdata) {
	&PrepareStaticReport;
	print "Done.\n" if $verbose;
    }
    else {
	die "No data available to build report. Check your log file and command line options.\nTry running AccessWatch with \$compressdata = 0 in the .cfg file.\nAlso, be sure to run aw-parser first.\n\nVisit http://accesswatch.com/help/ for more help.\n";
    }
}

__END__

